% ---------------------------------------------
% Author: Joaquin Cortes
% Copyright (c) 2012 University at Buffalo
%
% See the file "LICENSE.txt" for information on 
% usage and redistribution of this file, and for 
% a DISCLAIMER OF ALL WARRANTIES.
% ---------------------------------------------
%
% This is the Octave implementation of Tritone
% this tool generate ternary diagrams
% _______________________________________


%% read input from rappture fields ================================

% some useful stuff
allokay=1; %flag that needs to be passed in order to continue the algorithm
slash='/'; %forward slash for linux and mac machines
disp(' ');

% open our xml input file.
lib = rpLib(infile);

% retrieve user specified data about the input file 
howinput=rpLibGetString(lib,'input.group(load).choice(howinput).current');
if howinput=='fw'
   filein = rpLibGetString(lib,'input.group(load).string(filein).current');
   filein = strrep(filein,',','.'); % dealing with the latino decimal place
elseif howinput=='fl'
   datain = rpLibGetString(lib,'input.group(load).string(fileup).current');
   datain = strrep(datain,',','.'); % dealing with the latino decimal place
end

ifheader=rpLibGetString(lib,'input.group(load).boolean(ifheader).current');
if strcmp(ifheader,'yes')
   ifheader=1;
else
   ifheader=0;
end

ifsave=rpLibGetString(lib,'input.group(load).boolean(save).current');
if strcmp(ifsave,'yes')
    pathout= rpLibGetString(lib,'input.group(load).group(save).string(pathout).current');
    fileprefix= rpLibGetString(lib,'input.group(load).group(save).string(fileprefix).current');
    fileprefix= [fileprefix '_'];
else
    pathout='';
    fileprefix='';
end

% check that one of the options to get to the data has been selected
if howinput == 'fn'
   rpLibPutString(lib,'output.string(cmt).current','Please select a method to upload your data',0); 
   allokay=0;
end

% if want to plot legend
ifkey=rpLibGetString(lib,'input.group(key).boolean(ifkey).current');

% name of series
series=rpLibGetString(lib,'input.group(key).string(names).current');

%if custom symbols are required
ifsymbols=rpLibGetString(lib,'input.group(symbols).boolean(ifsymbols).current');

%symbols
symbols=rpLibGetString(lib,'input.group(symbols).string(symbols).current');

% set output comments
rpLibPutString(lib,'output.string(cmt).about.label','Output summary',0);

% size of the symbol
sizes=rpLibGetString(lib,'input.group(plot).choice(sizes).current');%1,2 or 3

  % Type of ternary diagram (default, afm, qapI, qapV, per)
  type=rpLibGetString(lib,'input.group(plot).choice(type).current');

% get username and path to user's account
%pathR='/home/'; %my debian machine
pathR='/home/vhub/'; %vhub's common user's path (Root)
pathL=length(pathR);
[tmp,pathPWD]=system('pwd');
pathA=pathPWD(pathL+1:length(pathPWD));
pathUser=strtok(pathA,slash);
pathUser=[pathR pathUser slash];

%% reads the file with input data ========================

if allokay && strcmp(ifsave,'yes') % make sure that output folder is okay

   if strcmp(pathout(1),slash) %if starts with slash, remove it
      pathout=pathout(2:length(pathout));
   end  
   if ~strcmp(pathout(length(pathout)),slash) %if doesn't end with slash, add it
      pathout=[pathout slash];
   end
   
   pathout=[pathUser pathout];
   disp(['Output path: ' pathout])
   
   [status,msg]=mkdir(pathout);
    
   if status==0
       rpLibPutString(lib,'output.string(cmt).current',['Error creating folder in the path specified: ' msg],0);    
       allokay=0;
   end
end

if allokay %opens the specified file with data in ascii format
      
   disp('Reading the input file...');

   % save input if data loaded in rappture window
   if howinput == 'fl'
      % save the uploaded data to a file
      filein=[pathout 'data.txt'];
      save("-text",'tmp.txt',"datain");
      system(['grep -v ^# tmp.txt > ' filein]);    %ifheader=ifheader+5; %octave adds 5 header lines when saving the txt file
   else 
      if strcmp(filein(1),slash) %if starts with slash, remove it
         filein=filein(2:length(filein));
      end  
      filein=[pathUser filein];
      disp(['Reading file: ' filein]);    
   end

%order of the columns to read

       axisX = rpLibGetString(lib,'input.group(plot).choice(Left).current');
       axisY = rpLibGetString(lib,'input.group(plot).choice(Top).current');
       axisZ = rpLibGetString(lib,'input.group(plot).choice(Right).current');  
       axisSort=[str2num(axisX) str2num(axisY) str2num(axisZ)];

if(sum(axisSort)!=6) %In case the axes are not different
  rpLibPutString(lib,'output.string(cmt).current','Selected axes have to be different for a meaningful plot',0);    
      allokay=0;
   end

   % let's open the file and read the matrix data
   [data,fid,msg]=read_txt(filein,ifheader);
   
   if fid<0 %in case there is problem opening that file
      rpLibPutString(lib,'output.string(cmt).current',['Error opening the input file: ' msg],0);    
      allokay=0;
   end
  
end 

%% save file with key labels ==============================
if allokay && strcmp(ifkey,'yes')
   keyfilename=[pathout fileprefix 'key'];
%   save("-text",[keyfilename '.txt'],"series");
   disp("Saving key file with series' names...");
   fid=fopen([keyfilename '.txt'],'w');
   fprintf(fid,'%s\n',series);
   fclose(fid);
end

%% plots the ternary diagram ==================================

if allokay
    
    tooldir=getenv('TOOLDIR');
    gpdir=[tooldir '/bin/']; %this one works in vhub's workspace; run the tool with ./middleware/invoke
%   gpdir='/home/jose/Dropbox/code/mytools/pingu/src/';
    
    pip=popen('gnuplot','w'); %open gnuplot pipe

    % Symbol selection
    
    if(strcmp(ifsymbols,'no'))
  
    switch sizes    
      case '1'
        fprintf(pip,"load '%sstyle1.gp'\n",gpdir); %set terminal and load styles
      case '2'
        fprintf(pip,"load '%sstyle2.gp'\n",gpdir); %set terminal and load styles
      case '3'
        fprintf(pip,"load '%sstyle3.gp'\n",gpdir); %set terminal and load styles
      otherwise
        fprintf(pip,"load '%sstyle1.gp'\n",gpdir); %set terminal and load styles
    endswitch
    
    else
        %sizes 1,2,3
     load_style(pip,symbols,sizes,pathout);   
    end
    
                 
%plotting part
       disp('plotting ternary diagram...');
	     %getting the axes labels and specific plot irv =0 for all
       irv=0;
    switch type
     case '1'
       ejeX = rpLibGetString(lib,'input.group(plot).string(value6).current');   
       ejeY = rpLibGetString(lib,'input.group(plot).string(value7).current');
       ejeZ = rpLibGetString(lib,'input.group(plot).string(value8).current');
     case '2'
       ejeX = 'A';
       ejeY = 'F';
       ejeZ = 'M';
       fprintf(pip,"load '%safmLabels.gp'\n",gpdir);
       irv=1;
     case '3'
       ejeX = 'A';
       ejeY = 'Q';
       ejeZ = 'P';
       fprintf(pip,"load '%sQAPigneous.gp'\n",gpdir);
     case '4'
       ejeX = 'A';
       ejeY = 'Q';
       ejeZ = 'P';
       fprintf(pip,"load '%sQAPVolc.gp'\n",gpdir);
     case '5'
       ejeX = 'Opx';
       ejeY = 'Ol';
       ejeZ = 'Cpx';
       fprintf(pip,"load '%speridot.gp'\n",gpdir);
    endswitch

    
       %Delete blank spaces from names of axis
       lejeX=ejeX;
       lejeY=ejeY;
       lejeZ=ejeZ;
       lejeX(lejeX== " ")="_";
       lejeY(lejeY== " ")="_";
       lejeZ(lejeZ== " ")="_";

       %wether we want or not grid
       grilla = rpLibGetString(lib,'input.group(plot).boolean(value9).current');
       
       plotname=[ejeX ejeY ejeZ ' plot'];
       plotfilename=[pathout fileprefix lejeX lejeY lejeZ];
       fprintf(pip,"load '%sgen3.gp'\n",gpdir);
       if  strcmp(grilla,'yes')
           fprintf(pip,"load '%sgrid.gp'\n",gpdir);
       end


       
       fprintf(pip,"set output '%s.eps'\n",plotfilename); %set name of output file
                                                          %and creates de diagram
       ns=gen3_outfile([plotfilename '.txt'],data,axisSort(1),axisSort(2),axisSort(3),lejeX,lejeY,lejeZ); %creates the file to use for plotting
         %but first add labels
       fprintf(pip,"set label '%s' at 0,-0.03 center\n",ejeX);
       fprintf(pip,"set label '%s' at 1,-0.03 center\n",ejeZ);
       fprintf(pip,"set label '%s' at 0.5, 0.9 center\n",ejeY);
       
       
       gen3_plot([plotfilename '.txt'],ns-1,pip,gpdir,irv); %do the plotting with gnuplot
       
	% creates a svg version
       fprintf(pip, "set terminal svg\n");
       fprintf(pip,"set output '%s.svg'\n",plotfilename);
       fprintf(pip, "replot\n");

   % makes the key (eps & svg)
   if strcmp(ifkey,'yes')
      key_plot(pip,keyfilename,ns,ifsymbols,gpdir,pathout); 
   end 

   pclose(pip);
  
   
   %convert eps to hig-res jpg
   disp('converting eps file to jpg format for display...');
   str=['convert -density 300 -rotate 90 ' plotfilename '.eps ' plotfilename '.jpg'];
   system(str);
   if strcmp(ifkey,'yes')
      str_key=['convert -density 300 -rotate 90 ' keyfilename '.eps ' keyfilename '.jpg'];   
      system(str_key);
   end
end
    
%% creates the rappture output

if allokay
    
   %Output summary
   rpLibPutString(lib,'output.string(cmt).current',"Files created:\n",0);
   if strcmp(howinput,'fl')
      rpLibPutString(lib,'output.string(cmt).current',[filein " Uploaded data file\n"],1);
   end
   rpLibPutString(lib,'output.string(cmt).current',[plotfilename ".eps Figure in postscript format\n"],1);
   rpLibPutString(lib,'output.string(cmt).current',[plotfilename ".svg Figure in svg format\n"],1);
   rpLibPutString(lib,'output.string(cmt).current',[plotfilename ".jpg Figure in jpeg format\n"],1);
   rpLibPutString(lib,'output.string(cmt).current',[plotfilename ".txt Text file with values plotted\n"],1);      
   if strcmp(ifkey,'yes')
      rpLibPutString(lib,'output.strint(cmt).curremt','',1);
      rpLibPutString(lib,'output.string(cmt).current',[keyfilename ".eps Figure with the key in postcript format\n"],1);
      rpLibPutString(lib,'output.string(cmt).current',[plotfilename ".svg Figure in svg format\n"],1);
      rpLibPutString(lib,'output.string(cmt).current',[keyfilename ".jpg Figure with the key in jpeg format\n"],1);
      rpLibPutString(lib,'output.string(cmt).current',[keyfilename ".txt Text file with the names of the series\n"],1);
   end
      
   %figure
   rpLibPutString(lib,'output.image(outvar).about.label',plotname,0);
   rpLibPutString(lib,'output.image(outvar).current','',0);
   rpLibPutFile(lib,'output.image(outvar).current',[plotfilename '.jpg'],1,0);
   if strcmp(ifkey,'yes')
     rpLibPutString(lib,'output.image(key).about.label','Key',0);
     rpLibPutFile(lib,'output.image(key).current',[keyfilename '.jpg'],1,0);
   end
end 

% signal the end of processing
rpLibResult(lib);

quit; %instead of return, 'quit' exits the octave session

